// pidemo.cpp : Defines the entry point for the DLL application.
//
// Image processing demo plugin for MSB Astroart (version 8.0 SP4 or later).

#include "stdafx.h"
#include "pidemo.h"
#pragma warning( disable : 4244 ) // double to float conversion


BOOL APIENTRY DllMain( HANDLE hModule, DWORD  ul_reason_for_call, 
                       LPVOID lpReserved)
{
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
		case DLL_THREAD_ATTACH:
		case DLL_THREAD_DETACH:
		case DLL_PROCESS_DETACH:
			break;
    }
    return TRUE;
}
  
enum AstroartCommand {ac_getbuffer, ac_getsize, ac_redraw, ac_close, ac_savefits,
   ac_copy, ac_writeheader, ac_readheader, ac_prepareundo, ac_thresholds, ac_transferfunc,
   ac_getselpoint, ac_getselpoint2, ac_gethwnd, ac_create, ac_open, ac_getimage,
   ac_getversion, ac_rename, ac_getname, ac_getra, ac_getdec,ac_getx, ac_gety, ac_modified,
   ac_blink, ac_flip, ac_resize, ac_findcoordinates, ac_reserved_1, ac_selectpoint, ac_selectrect, 
   ac_reserved_2, ac_reserved_3, ac_reserved_4, ac_centeratlas, ac_displayhelp, ac_save8bit,
   ac_preprocessing, ac_executemacro, ac_showdialog, ac_getdialogparam, ac_getundobuffer
};


typedef intptr_t (WINAPI *Tfcallback)(AstroartCommand, void*, void*, void*);  

Tfcallback fcallback;

int PC_GREEN = 1;
int PC_RED = 2;
int PC_BLUE = 3;

int PD_PREVIEW = 1;
int PD_PREVIEW_REALTIME = 2;


// ------------------------ The four functions to be implemented --------------------------------

extern "C" 
int WINAPI pi_initialize(Tfcallback funcaddress, char *menuname, char *description, HWND whandle)
{
 strcpy(menuname,"SDK demo plugin");
 strcpy(description,"Demo plugin in C++, a high pass filter");
 fcallback = funcaddress;
 return 1;
}

extern "C" 
void WINAPI pi_finalize(HWND whandle)
{
 // optional clean-up
}

char *DLG_NAME = "SDK Demo";

extern "C" 
void WINAPI pi_dialog(void *pimage, HWND whandle)
{
 char *params = "NUMBER \"High pass\" 70 0 100 5 \n"
                "BOOLEAN \"Flip image\"  0 \n";
 int flags = PD_PREVIEW + PD_PREVIEW_REALTIME;
 fcallback(ac_showdialog, DLG_NAME, params, (void*)flags);
}


int dimx,dimy;
float *newbuffer, *undobuffer;

float getPixel(int x, int y)
{
 if (x<0) x = 0;
 if (y<0) y = 0;
 if (x>=dimx) x = dimx-1;
 if (y>=dimy) y = dimy-1;
 return undobuffer[x+y*dimx];
}

void setPixel(int x, int y, float v)
{
 if (x<0 || y<0 || x >= dimx || y >= dimy) return;
 newbuffer[x+y*dimx] = v;
}

extern "C" 
void WINAPI pi_process(void *pimage, HWND whandle)
{
 double *pv1,*pv2;	 
 double vnew,vori,c1,c2;
 int x,y,ix,iy,r;
 bool cflip;

 pv1 = (double*)fcallback(ac_getdialogparam, DLG_NAME, (void*)1, NULL);
 c1 = *pv1 / 100.0;
 c2 = 1.0 - c1;
 pv2 = (double*)fcallback(ac_getdialogparam, DLG_NAME, (void*)2, NULL);
 cflip = *pv2 != 0.0;

 fcallback(ac_getsize, pimage, &dimx, &dimy);
 
 r = fcallback(ac_getbuffer, pimage, &newbuffer,(void*)PC_GREEN);  
 if (r != 2) return; 
 r = fcallback(ac_getundobuffer, pimage, &undobuffer,(void*)PC_GREEN);  
 if (r != 2) return;
 for (iy=0; iy<dimy; iy++) {                   
   y = iy;
   if (cflip)
     y = dimy-1-iy;
   for (ix=0; ix<dimx; ix++) {                                                            
     x = ix;
	 vnew = getPixel(x,y)*5.0 - getPixel(x+1,y) - getPixel(x,y+1) - getPixel(x-1,y) - getPixel(x,y-1);
     vori = getPixel(x,y);
     setPixel(ix, iy, vnew*c1 + vori*c2);
   }
 }
 // on color images you should repeat this block for the other two color planes,
 // (PC_RED, PC_BLUE), otherwise only the green plane will be processed.
 // You could use a cycle: for i = PC_GREEN to PC_BLUE ...
 // To check if an image is grayscale compare the pointer of PC_GREEN to PC_RED or PC_BLUE,
 // if they are identical then the image is grayscale.
 // e.g.
 // fcallback(ac_getbuffer, pimage, &greenbuffer, PC_GREEN);
 // fcallback(ac_getbuffer, pimage, &redbuffer, PC_RED);
 // if greenbuffer == redbuffer then the image is grayscale
}
